import 'dart:io';

import 'package:geiger_localstorage/geiger_localstorage.dart';
import 'package:test/test.dart';

class UserTest {}

void luongTests() {
  test('20210929-test1', () {
    StorageController geigerToolboxStorageController =
        GenericController('MI-Cyberrange', SqliteMapper('./dbFileName.sqlite'));
    geigerToolboxStorageController.close();
    File('./dbFileName.sqlite').deleteSync();
  });
  test('20210929-test2', () {
    // init
    StorageController geigerToolboxStorageController =
        GenericController('MI-Cyberrange', DummyMapper());

    // sample code
    var newScore = 123;
    var currentUserLevel = 123;
    try {
      Node node = geigerToolboxStorageController.get(':score-node');
      // this is bad coding as it depends on a full transaction database state. Introduced addOrUpdateValue as drop in replacement.
      node.addValue(NodeValueImpl('score', '$newScore'));
      node.addValue(NodeValueImpl('level', '$currentUserLevel'));
      geigerToolboxStorageController.update(node);
    } catch (e) {
      print('Exception while creating node');
      print(e.toString());

      Node node = NodeImpl('score-node');
      geigerToolboxStorageController.add(node);
      node.addValue(NodeValueImpl('score', '$newScore'));
      node.addValue(NodeValueImpl('level', '$currentUserLevel'));
      geigerToolboxStorageController.update(node);
    }
  });
}

class Event {
  final EventType _event;
  final Node? _old;
  final Node? _new;

  Event(this._event, this._old, this._new);

  EventType get type => _event;

  Node? get oldNode => _old;

  Node? get newNode => _new;

  @override
  String toString() {
    return '${type.toString()} ${oldNode.toString()}=>${newNode.toString()}';
  }
}

class CustomStorageListener implements StorageListener {
  List<Event> events = <Event>[];

  @override
  void gotStorageChange(EventType event, Node? oldNode, Node? newNode) {
    Event e = Event(event, oldNode, newNode);
    events.add(e);
    print('got event ${e.toString()}');
  }

  Future<List<Event>> getNumEvents(int num,[int timeout=1000000]) async {
    return Future.doWhile(() => events.length < num).timeout(Duration(seconds: timeout), onTimeout: () {
      throw ('Timeout');}).then((value) => events);
  }
}

void nosaTests() {
  test(
      '20211012-flutter Github Issue 16 (https://github.com/cyber-geiger/toolbox-storage/issues/16)',
      () async {
    StorageController _storageController =
        GenericController("Example", SqliteMapper("./database.db"));
    _storageController.zap();
    CustomStorageListener l = CustomStorageListener();
    SearchCriteria s = SearchCriteria(':', '%', '%');
    _storageController.registerChangeListener(l, s);
    _storageController.add(NodeImpl(':hello'));
    List<Event> e =
        await l.getNumEvents(1,3);
    expect(e.length,1, reason: 'Did not get expected number of results');
  });
}

void main() {
  luongTests();
  nosaTests();
}
